#!/bin/sh
#------------------------------------------------------------------------------
# syslog.cgi - show the syslog-file
#
# Creation:     24.02.2001  tg
# Last Update:  $Id: log_syslog.cgi 36698 2015-01-26 07:48:56Z kristov $
#
#------------------------------------------------------------------------------

# get main helper functions
. /srv/www/include/cgi-helper

# Functions
get_file_from_target ()
{
    set -f    # disable pathname expansion
    while read a b
    do
        if [ "$FORM_target" = "$a" -a -f "$b" ]
        then
            syslogfile=$b
        fi
    done < /etc/syslog.conf
    set +f
}

dump_log ()
{
    case $FORM_filter in
        "") cat $syslogfile ;;
        *) grep -v $FORM_filter $syslogfile ;;
    esac
}

format_output ()
{
    htmlspecialchars | while read line
    do
        case "$line" in
            '') ;;
            *)
                set -f
                set -- $line
                def="<td>$1 $2</td><td>$3</td><td>$4</td>"
                case $5$6$7 in
                    "--MARK--"*|"lastmessagerepeated"*)
                        def="$def<td>&nbsp;</td>"
                        sn=4
                        facil="mark"
                    ;;
                    kernel*)
                        def="$def<td>$5&nbsp;$6</td>"
                        sn=6
                        facil="$6"
                    ;;
                    *)
                        def="$def<td>$5</td>"
                        sn=5
                        facil="$5"
                    ;;
                esac
                shift $sn
                echo "<tr class=\"$facil\">$def<td>$*</td></tr>"
                set +f
            ;;
        esac
    done
}

: ${FORM_action:=view}

# Security
check_rights "logs" "$FORM_action"

if [ ! -f /etc/syslog.conf ]
then
    show_html_header "$_MN_err"
    show_error "$_MN_err" "$_SLOG_nosyslog"
    show_html_footer
else
    case $FORM_action in
    reset|$_MN_reset)
        get_file_from_target
        case $syslogfile in
            "") ;;
            *)
                > $syslogfile
            ;;
        esac
        echo "Location: $myname?target=$(echo "$FORM_target" | url_encode)"
        echo
    ;;
    download|$_MN_download)
        get_file_from_target
        case $syslogfile in "");;
            *)
                http_header download "ctype=text/plain;filename=`basename $syslogfile`"
                dump_log
            ;;
        esac
    ;;
    *)
        # standard sort method is normal
        : ${FORM_sort:=normal}
        case $FORM_sort in
            reverse)
                invsort="normal"
                ;;
            *)
                FORM_sort="normal" # make sure we don't have to escape $FORM_sort
                invsort="reverse"
                ;;
        esac
        # show all rows if the "all rows" button is pressed
        case $FORM_showall in "") ;; *) FORM_lines="$_MN_all" ;; esac
        # sanitize no. of rows: show the default of 50 rows if nothing or nonsens (no integer)  is specified
        case $FORM_lines in all|"$_MN_all") ;; *[^0-9]*|"") FORM_lines=50 ;; esac
        while read target to
        do
            if [ -f "$to" ]
            then
                case $FORM_target in
                ""|"$target") 
                    tabs="${tabs}\"$_SLOG_syslogfile: $to\" no "
                    FORM_target="$target"
                ;;
                *)
                    tabs="${tabs}\"$_SLOG_syslogfile: $to\" \"$myname?target=$target&amp;lines=$FORM_lines&amp;sort=$FORM_sort\" "
                ;;
                esac
            fi
        done < /etc/syslog.conf
        show_html_header "Syslog"
        get_file_from_target
        case $syslogfile in
            "") show_error "$_MN_err" "$_SLOG_nofile ($(echo "$FORM_target" | htmlspecialchars))" ;;
            *)
                case $FORM_lines in
                    all|"$_MN_all")
                        # is "all"
                        n="$_MN_all"
                        case $FORM_sort in 
                            normal)
                                dump_log | format_output > /tmp/sysout.$$
                            ;;
                            *)
                                dump_log | sed '1!G;h;$!d' | format_output > /tmp/sysout.$$
                            ;;
                        esac
                    ;;
                    *)
                        # is an integer
                        n=$FORM_lines
                        case $FORM_sort in 
                            normal)
                                dump_log | do_tail $n | format_output > /tmp/sysout.$$
                            ;;
                            *)
                                dump_log | do_tail $n | sed '1!G;h;$!d' | format_output > /tmp/sysout.$$
                            ;;
                        esac
                    ;;
                esac
                eval show_tab_header $tabs
                echo "<h2>$_SLOG_syslogfile: $syslogfile ($(echo "$FORM_target" | htmlspecialchars))</h2>"
                # Display form to change number of shown rows of logfile
                cat <<-EOF
	<form action="$myname" method="GET">
	    <input class="itext" type="text" size="3" maxlength="4" name="lines" value="$FORM_lines">&nbsp;$_MN_rows&nbsp;&nbsp;
	    <input class="sbutton" type="submit" value="$_MN_show">
	    <input class="sbutton" type="submit" name="showall" value="$_MN_BTN_showallrows">
	    <input class="sbutton" type="submit" name="action" value="$_MN_download">
	    <input class="sbutton" type="submit" name="action" value="$_MN_reset">
	    <input type="hidden" name="target" value="$(echo "$FORM_target" | htmlspecialchars quot)">
	    <input type="hidden" name="sort" value="$FORM_sort">
	    <br>
	</form>
EOF
                echo "<table class=\"normtable\"><tr><th><a href=\"$myname?target=$(echo "$FORM_target" | url_encode)&amp;lines=$FORM_lines&amp;sort=$invsort\">$_MN_date<img src=\"../img/s$invsort.gif\" alt=\" (sort $invsort)\"></a></th><th>$_MN_time</th><th>$_SLOG_loghost</th><th>$_SLOG_syslogfacil</th><th>$_SLOG_message</th></tr>"
                cat  /tmp/sysout.$$
                rm /tmp/sysout.$$ 
                echo '</table>'

                show_tab_footer
            ;;
        esac
        show_html_footer
    ;;
    esac
fi
